/*
 * MainFrame.java
 *
 * Created on Nov 23, 2012, 10:44:13 PM
 */
package demoviewer.gui;

import demoviewer.model.Demo;
import demoviewer.panels.DemoTreePanel;
import demoviewer.panels.FragTablePanel;
import demoviewer.panels.MeansOfDeathTablePanel;
import demoviewer.panels.SpreePanel;
import demoviewer.panels.TabButtonPanel;
import demoviewer.utils.FileUtils;
import demoviewer.utils.StringUtils;
import java.awt.BorderLayout;
import java.util.ArrayList;
import java.util.List;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import javax.swing.UIManager;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 *
 * @author Me
 */
public class MainFrame extends javax.swing.JFrame {
    
    private static final ImageIcon ICON = new ImageIcon(MainFrame.class.getResource("/demoviewer/resources/icons/film_big.png"));
    private static final ImageIcon ICON_CROSS_RED = new ImageIcon(MainFrame.class.getResource("/demoviewer/resources/icons/cross_red.png"));
    private static final ImageIcon ICON_TICK_RED = new ImageIcon(MainFrame.class.getResource("/demoviewer/resources/icons/tick_red.png"));
        
    public static final int TAB_COUNT_DEFAULT = 3;
    public static final int TAB_COUNT_MAX_DEMOS = 10;
    public static final int TAB_COUNT_MAX = TAB_COUNT_MAX_DEMOS + TAB_COUNT_DEFAULT;
    
    private static MainFrame the = null;
    
    private DemoTreePanel demoTreePanel = null;
    private SpreePanel spreePanel = null;
    private MeansOfDeathTablePanel meansOfDeathTablePanel = null;
    
    private List<FragTablePanel> fragTablePanels = null;
    
    private boolean filter = true;
        
    private MainFrame() {
        initComponents();
        init();
    }
    
    public static MainFrame the() {
        
        if (the == null) {
            the = new MainFrame();
        }
        
        return the;        
    }
    
    private void init() {
        
        setIconImage(ICON.getImage());
        
        demoTreePanel = new DemoTreePanel();
        jpDemos.add(demoTreePanel, BorderLayout.CENTER);
        
        spreePanel = new SpreePanel();
        jpSprees.add(spreePanel, BorderLayout.CENTER);
        
        meansOfDeathTablePanel = new MeansOfDeathTablePanel();
        jpWeapons.add(meansOfDeathTablePanel, BorderLayout.CENTER);
        
        TabButtonPanel demoTabButtonPanel = new TabButtonPanel(tabbedPane, 0, false);
        tabbedPane.setTabComponentAt(0, demoTabButtonPanel); 
        
        TabButtonPanel filterTabButtonPanel = new TabButtonPanel(tabbedPane, 1, false);
        tabbedPane.setTabComponentAt(1, filterTabButtonPanel); 
        
        TabButtonPanel playerListTabButtonPanel = new TabButtonPanel(tabbedPane, 2, false);
        tabbedPane.setTabComponentAt(2, playerListTabButtonPanel); 
        
        fragTablePanels = new ArrayList<>();
        
        tabbedPane.addChangeListener(new ChangeListener() {
            @Override
            public void stateChanged(ChangeEvent e) {
                                
                if (tabbedPane.getSelectedIndex() >= TAB_COUNT_DEFAULT) {

                    int index = tabbedPane.getSelectedIndex() - TAB_COUNT_DEFAULT;
                    if (fragTablePanels != null && fragTablePanels.size() > index) {
                        fragTablePanels.get(index).setFilterMode(filter);
                    }
                }
            }
        });
    }
    
    public void viewDemo(Demo demo) {

        if (tabbedPane.getTabCount() == TAB_COUNT_MAX) {
            MainFrame.the().showErrorMessageDialog("You can only have " + TAB_COUNT_MAX_DEMOS + " demos open at once.");
        }
        else {

            FragTablePanel fragTablePanel = new FragTablePanel(demo);
            tabbedPane.add("  " + FileUtils.getFileNameWithoutExtension(demo.getFile()) + "  ", fragTablePanel);
            tabbedPane.setSelectedIndex(tabbedPane.getTabCount() - 1);

            TabButtonPanel tabButtonPanel = new TabButtonPanel(tabbedPane, tabbedPane.getTabCount() - 1, true);
            tabbedPane.setTabComponentAt(tabbedPane.getTabCount() - 1, tabButtonPanel);

            fragTablePanels.add(fragTablePanel);
        }
    }
    
    private void toggleDisableFilters() {
        
        setFilter(!filter);
        if (filter) {
            jmiFilter.setText("Disable");            
        }
        else {
            jmiFilter.setText("Enable");
        }
        
        updateFragTablePanels();
    }
    
    private void updateFragTablePanels() {
        
        for (FragTablePanel fragTablePanel : fragTablePanels) {
            fragTablePanel.setFilterMode(filter);
        }
    }
         
    public int getMinSpree() {
        return spreePanel.getMinSpree();
    }
    
    public int getTimeBetweenFrags() {
        return spreePanel.getTimeBetweenFrags();
    }
    
    public void showErrorMessageDialog(String message) {        
        JOptionPane.showMessageDialog(MainFrame.the(), message, "Error", JOptionPane.ERROR_MESSAGE);  
    }
    
    public void showErrorMessageDialog(String message, Throwable t) {        
        
        t.printStackTrace();
        
        StringBuilder sb = new StringBuilder();
        sb.append(message);
        
        if (!StringUtils.isNullOrEmpty(t.getMessage())) {
            sb.append("\n");
            sb.append(t.getMessage());
        }
        
        JOptionPane.showMessageDialog(MainFrame.the(), sb.toString(), "Error", JOptionPane.ERROR_MESSAGE);  
    }
        
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jpHolder = new javax.swing.JPanel();
        tabbedPane = new javax.swing.JTabbedPane();
        jpDemos = new javax.swing.JPanel();
        jpPlayList = new javax.swing.JPanel();
        jpFilters = new javax.swing.JPanel();
        jpSprees = new javax.swing.JPanel();
        jpWeapons = new javax.swing.JPanel();
        menuBar = new javax.swing.JMenuBar();
        jMenu1 = new javax.swing.JMenu();
        jMenuItem1 = new javax.swing.JMenuItem();
        jMenu2 = new javax.swing.JMenu();
        jMenuItem2 = new javax.swing.JMenuItem();
        jmGlobalFilters = new javax.swing.JMenu();
        jmiFilter = new javax.swing.JMenuItem();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("DemoViewer");

        jpHolder.setBorder(javax.swing.BorderFactory.createEmptyBorder(5, 5, 5, 5));
        jpHolder.setLayout(new java.awt.BorderLayout());

        tabbedPane.setFocusable(false);

        jpDemos.setBorder(javax.swing.BorderFactory.createEmptyBorder(5, 5, 5, 5));
        jpDemos.setPreferredSize(new java.awt.Dimension(512, 0));
        jpDemos.setLayout(new java.awt.BorderLayout());
        tabbedPane.addTab("  Demos  ", jpDemos);
        tabbedPane.addTab("  Play List  ", jpPlayList);

        jpFilters.setBorder(javax.swing.BorderFactory.createEmptyBorder(5, 5, 5, 5));
        jpFilters.setLayout(new java.awt.BorderLayout(0, 5));

        jpSprees.setPreferredSize(new java.awt.Dimension(10, 60));
        jpSprees.setLayout(new java.awt.BorderLayout());
        jpFilters.add(jpSprees, java.awt.BorderLayout.NORTH);

        jpWeapons.setLayout(new java.awt.BorderLayout());
        jpFilters.add(jpWeapons, java.awt.BorderLayout.CENTER);

        tabbedPane.addTab("  Filters  ", jpFilters);

        jpHolder.add(tabbedPane, java.awt.BorderLayout.CENTER);

        getContentPane().add(jpHolder, java.awt.BorderLayout.CENTER);

        jMenu1.setText("File");

        jMenuItem1.setText("jMenuItem1");
        jMenu1.add(jMenuItem1);

        menuBar.add(jMenu1);

        jMenu2.setText("Edit");

        jMenuItem2.setText("jMenuItem2");
        jMenu2.add(jMenuItem2);

        menuBar.add(jMenu2);

        jmGlobalFilters.setText("Filter");

        jmiFilter.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F, java.awt.event.InputEvent.ALT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        jmiFilter.setText("Disable");
        jmiFilter.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiFilterActionPerformed(evt);
            }
        });
        jmGlobalFilters.add(jmiFilter);

        menuBar.add(jmGlobalFilters);

        setJMenuBar(menuBar);

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-720)/2, (screenSize.height-580)/2, 720, 580);
    }// </editor-fold>//GEN-END:initComponents

    private void jmiFilterActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiFilterActionPerformed
        toggleDisableFilters();
    }//GEN-LAST:event_jmiFilterActionPerformed

    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {

        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
               
        java.awt.EventQueue.invokeLater(new Runnable() {

            @Override
            public void run() {    
                MainFrame.the();   
                MainFrame.the().setVisible(true);
            }
        });       
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenu jMenu1;
    private javax.swing.JMenu jMenu2;
    private javax.swing.JMenuItem jMenuItem1;
    private javax.swing.JMenuItem jMenuItem2;
    private javax.swing.JMenu jmGlobalFilters;
    private javax.swing.JMenuItem jmiFilter;
    private javax.swing.JPanel jpDemos;
    private javax.swing.JPanel jpFilters;
    private javax.swing.JPanel jpHolder;
    private javax.swing.JPanel jpPlayList;
    private javax.swing.JPanel jpSprees;
    private javax.swing.JPanel jpWeapons;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JTabbedPane tabbedPane;
    // End of variables declaration//GEN-END:variables

    public boolean isFilter() {
        return filter;
    }

    public void setFilter(boolean filter) {
        this.filter = filter;
    }
}
