/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package demoviewer.model;

import demoviewer.model.database.DatabaseConnector;
import demoviewer.model.database.ResultSetWrapper;
import demoviewer.model.label.ShadowLabel;
import java.io.File;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author Me
 */
public class Demo {
    
    public static final String EXTENSION = ".dm_60";
    
    private static List<ShadowLabel> fragLabels = null;
    
    private int id = -1;
    private String demoPath = "";
    private int demoPov = -1;
    
    private File file = null;
    private boolean scanning = false;
    private boolean scanned = false;
    
    private List<Player> players = null;
    private List<Frag> frags = null;
        
    private Demo() {        
    }
    
    public Demo(File file) {        
        this.file = file;  
    }
    
    public static Demo loadDemoFromDB(Demo demo) throws SQLException, ClassNotFoundException {
                
        ResultSetWrapper rsw = null;
        
        try {
            
            Connection connection = DatabaseConnector.getConnection();
            connection.setAutoCommit(false);
            PreparedStatement ps = connection.prepareStatement("select id, demo_path, demo_pov from demo where demo_path = ?;");
            ps.setString(1, demo.getFile().getAbsolutePath());
            ResultSet resultSet = ps.executeQuery();
            rsw = new ResultSetWrapper(connection, resultSet);
            
            while (resultSet.next()) {                
                demo.initFromResultSet(resultSet);                
            }            
        }
        finally {
            
            if (rsw != null) {
                rsw.close();
            }            
        }

        return demo; 
    }
    
    private void initFromResultSet(ResultSet resultSet) throws SQLException, ClassNotFoundException {
        
        id = resultSet.getInt("id");
        demoPath = resultSet.getString("demo_path");
        demoPov = resultSet.getInt("demo_pov");
        
        file = new File(demoPath);
        scanning = false;
        scanned = true;
    }
    
    public void loadPlayersAndFrags() throws SQLException, ClassNotFoundException {
        
        players = Player.getPlayersForDemoId(id);
        frags = Frag.getFragsForDemoId(id);
    }
    
    private void initFragLabels() {
        
        fragLabels = new ArrayList<>();
        
        for (Frag frag : getFrags()) {          
            ShadowLabel shadowLabel = new ShadowLabel();
            shadowLabel.setText(frag.getMessage()); 
            fragLabels.add(shadowLabel);
        }        
    }
    
    public boolean isPov(Player player) {
        
        if (demoPov == player.getPlayerId()) {
            return true;
        }

        return false;
    }
    
    public int getId() {
        return id;
    }

    public void setId(int id) {
        this.id = id;
    }
    
    public String getDemoPath() {
        return demoPath;
    }

    public void setDemoPath(String demoPath) {
        this.demoPath = demoPath;
    }
    
    public int getDemoPov() {
        return demoPov;
    }

    public void setDemoPov(int demoPov) {
        this.demoPov = demoPov;
    }
        
    public File getFile() {
        return file;
    }
    
    public void setFile(File file) {
        this.file = file;
    }   
    
    public boolean isScanning() {
        return scanning;
    }

    public void setScanning(boolean scanning) {
        this.scanning = scanning;
    }
    
    public boolean isScanned() {
        return scanned;
    }

    public void setScanned(boolean scanned) {
        this.scanned = scanned;
    }  
    
    public List<Player> getPlayers() {
        return players;
    }

    public void setPlayers(List<Player> players) {
        this.players = players;
    }
    
    public List<Frag> getFrags() {
        
        if (frags == null) {
            frags = new ArrayList<>();
        }
        
        return frags;
    }

    public void setFrags(List<Frag> frags) {
        this.frags = frags;
        initFragLabels();
    }
}
