/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package demoviewer.model;

import demoviewer.model.database.DatabaseConnector;
import demoviewer.model.database.ResultSetWrapper;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import javax.swing.ImageIcon;

/**
 *
 * @author Me
 */
public class Player implements Comparable<Player> {
    
    public static final ImageIcon IMAGE_ICON_ALLIES = new ImageIcon(Player.class.getResource("/demoviewer/resources/icons/allies.png"));
    public static final ImageIcon IMAGE_ICON_AXIS = new ImageIcon(Player.class.getResource("/demoviewer/resources/icons/axis.png"));
   
    public static final int TEAM_FREE = 0;
    public static final int TEAM_RED = 1;
    public static final int TEAM_BLUE = 2;
    public static final int TEAM_SPECTATOR = 3;
    
    private int id = -1;
    private int demoId = -1;
    private int playerId = -1;
    private String playerName = "";
    private int team = -1;
    
    private boolean selected = true;

    public Player() {
    }

    public Player(int id, int demoId, int playerId, String playerName, int team) {

        this.id = id;
        this.demoId = demoId;
        this.playerId = playerId;
        this.playerName = playerName;
        this.team = team;
    }
    
    public static List<Player> getPlayersForDemoId(int demoId) throws SQLException, ClassNotFoundException {
                
        List<Player> players = new ArrayList<>();
        
        ResultSetWrapper rsw = null;
        
        try {
            
            Connection connection = DatabaseConnector.getConnection();
            connection.setAutoCommit(false);
            PreparedStatement ps = connection.prepareStatement("select * from player where demo_id = ?;");
            ps.setInt(1, demoId);
            ResultSet resultSet = ps.executeQuery();
            rsw = new ResultSetWrapper(connection, resultSet);
            
            while (resultSet.next()) {
                
                Player player = new Player();
                player.initFromResultSet(resultSet);                
                players.add(player);
            }            
        }
        finally {
            
            if (rsw != null) {
                rsw.close();
            }            
        }
        
        Collections.sort(players);

        return players;        
    }
    
    private void initFromResultSet(ResultSet resultSet) throws SQLException {
        
        id = resultSet.getInt("id");
        demoId = resultSet.getInt("demo_id");
        playerId = resultSet.getInt("player_id");
        playerName = resultSet.getString("player_name");
        team = resultSet.getInt("team");  
    }

    public int getId() {
        return id;
    }

    public void setId(int id) {
        this.id = id;
    }

    public int getDemoId() {
        return demoId;
    }

    public void setDemoId(int demoId) {
        this.demoId = demoId;
    }

    public int getPlayerId() {
        return playerId;
    }

    public void setPlayerId(int playerId) {
        this.playerId = playerId;
    }

    public String getPlayerName() {
        return playerName;
    }
    
    public String getPlayerName(Demo demo) {
        
        if (demo.getDemoPov() == playerId) {
            return "^3^7" + playerName;
        }
        
        return playerName;
    }

    public void setPlayerName(String playerName) {
        this.playerName = playerName;
    }

    public int getTeam() {
        return team;
    }

    public void setTeam(int team) {
        this.team = team;
    }
    
    public String getTeamAsString() {
        
        if (team == TEAM_RED) {
            return "^1Axis";
        }
        else if (team == TEAM_BLUE) {
            return "^4Allies";
        }
        else if (team == TEAM_SPECTATOR) {
            return "^3Spectator";
        }
        
        return "Unknown";
    }
    
    public boolean isSelected() {
        return selected;
    }

    public void setSelected(boolean selected) {
        this.selected = selected;
    }
    
    @Override
    public String toString() {        
        return "Player: (" + id + ", " + demoId + ", " + playerId + ", " + playerName + ", " + team + ")";        
    }

    @Override
    public int compareTo(Player o) {
        
        if (team == o.getTeam()) {
            return 0;
        }
        
        if (team == TEAM_BLUE && (o.getTeam() == TEAM_RED || o.getTeam() == TEAM_SPECTATOR)) {
            return -1;
        }
        
        if (team == TEAM_RED && o.getTeam() == TEAM_BLUE) {
            return 1;
        }
        else if (team == TEAM_RED && o.getTeam() == TEAM_SPECTATOR) {
            return -1;
        }
        
        return 1;
    }
}
