package demoviewer.model.label;

import demoviewer.common.Fonts;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

/**
 *
 * @author Me
 */
public class ShadowLabel extends JLabel {
    
    private static final Color COLOR_SHADOW = new Color(0, 0, 0, 75);

    private static final int X_OFFSET = 2;
    private static final int Y_OFFSET = 2;
    
    private static final HashMap<Character, Color> colorMap = new HashMap<>();
    
    static {
        colorMap.put('0', new Color(0x000000));
        colorMap.put('P', new Color(0x000000));
        colorMap.put('p', new Color(0x000000));

        colorMap.put('1', new Color(0xFF0000));
        colorMap.put('Q', new Color(0xFF0000));
        colorMap.put('q', new Color(0xFF0000));

        colorMap.put('2', new Color(0x00FF00));
        colorMap.put('R', new Color(0x00FF00));
        colorMap.put('r', new Color(0x00FF00));

        colorMap.put('3', new Color(0xFFFF00));
        colorMap.put('S', new Color(0xFFFF00));
        colorMap.put('s', new Color(0xFFFF00));

        colorMap.put('4', new Color(0x0000FF));
        colorMap.put('T', new Color(0x0000FF));
        colorMap.put('t', new Color(0x0000FF));

        colorMap.put('5', new Color(0x00FFFF));
        colorMap.put('U', new Color(0x00FFFF));
        colorMap.put('u', new Color(0x00FFFF));

        colorMap.put('6', new Color(0xFF00FF));
        colorMap.put('V', new Color(0xFF00FF));
        colorMap.put('v', new Color(0xFF00FF));

        colorMap.put('7', new Color(0xFFFFFF));
        colorMap.put('W', new Color(0xFFFFFF));
        colorMap.put('w', new Color(0xFFFFFF));

        colorMap.put('8', new Color(0xFF7F00));
        colorMap.put('X', new Color(0xFF7F00));
        colorMap.put('x', new Color(0xFF7F00));

        colorMap.put('9', new Color(0x7F7F7F));
        colorMap.put('Y', new Color(0x7F7F7F));
        colorMap.put('y', new Color(0x7F7F7F));

        colorMap.put(':', new Color(0xBFBFBF));
        colorMap.put('Z', new Color(0xBFBFBF));
        colorMap.put('z', new Color(0xBFBFBF));
        colorMap.put(';', new Color(0xBFBFBF));
        colorMap.put('[', new Color(0xBFBFBF));
        colorMap.put('{', new Color(0xBFBFBF));

        colorMap.put('<', new Color(0x007F00));
        colorMap.put('|', new Color(0x007F00));

        colorMap.put('=', new Color(0x7F7F00));
        colorMap.put(']', new Color(0x7F7F00));
        colorMap.put('}', new Color(0x7F7F00));

        colorMap.put('>', new Color(0x00007F));
        colorMap.put('~', new Color(0x00007F));

        colorMap.put('?', new Color(0x7F0000));
        colorMap.put('_', new Color(0x7F0000));

        colorMap.put('@', new Color(0x7F3F00));

        colorMap.put('!', new Color(0xFF9919));
        colorMap.put('A', new Color(0xFF9919));
        colorMap.put('a', new Color(0xFF9919));

        colorMap.put('\"', new Color(0x007F7F));
        colorMap.put('B', new Color(0x007F7F));
        colorMap.put('b', new Color(0x007F7F));

        colorMap.put('#', new Color(0x7F007F));
        colorMap.put('C', new Color(0x7F007F));
        colorMap.put('c', new Color(0x7F007F));

        colorMap.put('$', new Color(0x007fff));
        colorMap.put('D', new Color(0x007fff));
        colorMap.put('d', new Color(0x007fff));

        colorMap.put('%', new Color(0x7f00ff));
        colorMap.put('E', new Color(0x7f00ff));
        colorMap.put('e', new Color(0x7f00ff));

        colorMap.put('&', new Color(0x3399cc));
        colorMap.put('F', new Color(0x3399cc));
        colorMap.put('f', new Color(0x3399cc));

        colorMap.put('\"', new Color(0xccffcc));
        colorMap.put('G', new Color(0xccffcc));
        colorMap.put('g', new Color(0xccffcc));

        colorMap.put('(', new Color(0x006633));
        colorMap.put('H', new Color(0x006633));
        colorMap.put('h', new Color(0x006633));

        colorMap.put(')', new Color(0xff0033));
        colorMap.put('I', new Color(0xff0033));
        colorMap.put('i', new Color(0xff0033));

        colorMap.put('*', new Color(0xb21919));
        colorMap.put('J', new Color(0xb21919));
        colorMap.put('j', new Color(0xb21919));

        colorMap.put('+', new Color(0x993300));
        colorMap.put('K', new Color(0x993300));
        colorMap.put('k', new Color(0x993300));

        colorMap.put(',', new Color(0xcc9933));
        colorMap.put('L', new Color(0xcc9933));
        colorMap.put('l', new Color(0xcc9933));

        colorMap.put('-', new Color(0x999933));
        colorMap.put('M', new Color(0x999933));
        colorMap.put('m', new Color(0x999933));

        colorMap.put('.', new Color(0xffffbf));
        colorMap.put('N', new Color(0xffffbf));
        colorMap.put('n', new Color(0xffffbf));

        colorMap.put('/', new Color(0xffff7f));
        colorMap.put('O', new Color(0xffff7f));
        colorMap.put('o', new Color(0xffff7f));
    }

    private List<ConsoleChar> consoleChars = null;

    public ShadowLabel() {
        super();   
        init();
    }
    
    private void init() {
        setFont(Fonts.FONT_CONSOLE); 
        setOpaque(false);
        setBackground(new Color(0, 0, 0, 0));
    }

    @Override
    public void paintComponent(Graphics g) {

        Graphics2D graphics2D = (Graphics2D)g;
        
        graphics2D.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
        
        FontMetrics fm = getFontMetrics(getFont());

        int h = fm.getAscent();
        int x = 0;
        
        for (ConsoleChar consoleChar : consoleChars) {

            int w = fm.charWidth(consoleChar.getCharacter());

            g.setColor(COLOR_SHADOW);
            g.drawString("" + consoleChar.getCharacter(), x + X_OFFSET, h + Y_OFFSET);

            g.setColor(consoleChar.getColor());
            g.drawString("" + consoleChar.getCharacter(), x, h);

            x += w;
        }

        graphics2D.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_DEFAULT);
    }
    
    @Override
    public void setText(String text) {
        super.setText(text);
        initConsoleChars(text);
    }

    public void initConsoleChars(String text) {
        
        consoleChars = new ArrayList<>();
        
        char[] chars = text.toCharArray();

        boolean escapeChar = false;
        Color color = Color.WHITE;

        for (int i = 0; i < chars.length; i++) {

            if (escapeChar) {
                color = colorMap.get(chars[i]);
                escapeChar = false;
                i++;
            }
            else if (chars[i] == '^') {
                escapeChar = true;
            }

            if (!escapeChar && (i != chars.length) && chars[i] != '^') {
                consoleChars.add(new ConsoleChar(chars[i], color));
            }
            else if (!escapeChar && (i != chars.length) && chars[i] == '^') {
                i--;
            }
            else if (escapeChar && (i != chars.length - 1)) {

                i++;

                if (chars[i] != '^') {
                    i--;
                }
            }
        }
    }
    
    private class ConsoleChar {
        
        private char character = '\u0000';
        private Color color = null;
        
        public ConsoleChar(char character, Color color) {
            this.character = character;
            this.color = color;
        }
        
        public char getCharacter() {
            return character;
        }

        public void setCharacter(char character) {
            this.character = character;
        }

        public Color getColor() {
            return color;
        }

        public void setColor(Color color) {
            this.color = color;
        } 
    }

    public static void main(String[] args) {
        ShadowLabel label = new ShadowLabel();
        label.setText("^1test");
        // drop shadow w/ highlight
        //label.setLeftShadow(2, 2, Color.white);
        //label.setRightShadow(2, 2, new Color(0, 0, 0, 100));
        
        //label.setColoredText();        
        
        //label.setForeground(Color.gray);
        //label.setFont(label.getFont().deriveFont(14 of));
        label.setFont(new Font("Verdana", 1, 36));
        
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());
        
        panel.add(label, BorderLayout.CENTER);
        
        JFrame frame = new JFrame("Label");
        //frame.getContentPane().add(panel);
        frame.add(panel);
        frame.pack();
        frame.setVisible(true);
    }
}
