/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package demoviewer.model.tree;

import demoviewer.panels.DemoTreePanel;
import demoviewer.utils.FileUtils;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import javax.swing.tree.TreePath;
import org.jdesktop.swingx.treetable.AbstractTreeTableModel;

/**
 *
 * @author Me
 */
public class DemoTreeTableModel extends AbstractTreeTableModel {

    private static final String DATE_FORMAT = "MM/dd/yyyy HH:mm";
    private static final String[] COLUMN_NAMES = {"", "Date Modified", "Size", "Type", ""};
    
    private DemoTreePanel demoTreePanel = null;
    
    public DemoTreeTableModel(DemoTreePanel demoTreePanel) {
        super();
        this.demoTreePanel = demoTreePanel;
    }

    public DemoTreeTableModel(Object root, DemoTreePanel demoTreePanel) {
        super(root);
        this.demoTreePanel = demoTreePanel;
    }

    @Override
    public int getColumnCount() {
        return COLUMN_NAMES.length;
    }

    @Override
    public String getColumnName(int columnIndex) {
        return COLUMN_NAMES[columnIndex];
    }

    @Override
    public Class<?> getColumnClass(int columnIndex) {

        switch (columnIndex) {
            case 0:
                return Node.class;
            case 1:
                return String.class;
            case 2:
                return String.class;
            case 3:
                return String.class;
            case 4:
                return Boolean.class;
            default:
                return String.class;
        }
    }

    @Override
    public boolean isCellEditable(Object node, int columnIndex) {

        if (columnIndex == 4) {
            return !demoTreePanel.isScanning();
        }

        return false;
    }

    @Override
    public Object getValueAt(Object object, int columnIndex) {

        Node node = (Node)object;
        if (node instanceof DirectoryTreeNode) {
            DirectoryTreeNode directoryTreeNode = (DirectoryTreeNode)node;
            return getValueAtForDirectoryTreeNode(directoryTreeNode, columnIndex);
        }
        else if (node instanceof DemoTreeNode) {
            DemoTreeNode demoTreeNode = (DemoTreeNode)node;
            return getValueAtForDemoTreeNode(demoTreeNode, columnIndex);
        }

        return "";
    }

    public Object getValueAtForDirectoryTreeNode(DirectoryTreeNode directoryTreeNode, int columnIndex) {

        switch (columnIndex) {
            case 0:
                return directoryTreeNode;
            case 1:
                return "";
            case 2:
                return "";
            case 3:
                return "Demo folder";
            case 4:
                return directoryTreeNode.isChecked();
            default:
                return "";
        } 
    }

    public Object getValueAtForDemoTreeNode(DemoTreeNode demoTreeNode, int columnIndex) {

        switch (columnIndex) {
            case 0:
                return demoTreeNode;
            case 1:
                return getDateStringFromMillis(demoTreeNode.getDemo().getFile().lastModified());
            case 2:
                return FileUtils.getFileSizeAsString(demoTreeNode.getDemo().getFile());
            case 3:
                return FileUtils.getFileExtension(demoTreeNode.getDemo().getFile());
            case 4:
                return demoTreeNode.isChecked();
            default:
                return "";
        }
    }

    @Override
    public void setValueAt(Object value, Object currentNode, int columnIndex) {

        Node node = (Node)currentNode;
        if (node instanceof DirectoryTreeNode) {
            DirectoryTreeNode directoryTreeNode = (DirectoryTreeNode)node;
            setValueAtForDirectoryTreeNode(value, directoryTreeNode, columnIndex);
        }
        else if (node instanceof DemoTreeNode) {
            DemoTreeNode demoTreeNode = (DemoTreeNode)node;
            setValueAtForDemoTreeNode(value, demoTreeNode, columnIndex);
        }
    }

    public void setValueAtForDirectoryTreeNode(Object value, DirectoryTreeNode directoryTreeNode, int columnIndex) {

        if (columnIndex == 4) {

            if (value instanceof Boolean) {
                Boolean isChecked = (Boolean)value;
                directoryTreeNode.setChecked(isChecked);

                toggleCheckBoxes(directoryTreeNode, isChecked);
            }
        }
    }

    public void setValueAtForDemoTreeNode(Object value, DemoTreeNode demoTreeNode, int columnIndex) {

        if (columnIndex == 4) {

            if (value instanceof Boolean) {
                Boolean isChecked = (Boolean)value;
                demoTreeNode.setChecked(isChecked);
                
                toggleCheckBoxes(demoTreeNode, isChecked);
            }
        }
    }

    @Override
    public Object getChild(Object parent, int index) {

        Node node = (Node)parent;
        return node.getChildAt(index);
    }

    @Override
    public int getChildCount(Object parent) {

        Node node = (Node)parent;
        return node.getChildCount();
    }

    @Override
    public int getIndexOfChild(Object parent, Object child) {

        Node node = (Node)parent;
        Node childNode = (Node)child;
        return node.getIndex(childNode);
    }
    
    public void refreshRootNode() {

        Node rootNode = (Node)getRoot();
        TreePath treePath = new TreePath(rootNode.getPath());
        valueForPathChanged(treePath, rootNode);
    }
    
    public void refreshNode(Node node) {

        TreePath treePath = new TreePath(node.getPath());
        valueForPathChanged(treePath, node);
    }

    private Date getDateFromMillis(long millis) {
        return new Date(millis);
    }

    private String getDateStringFromDate(Date date) {

        SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT);
        return sdf.format(date);
    }

    private String getDateStringFromMillis(long millis) {
        return getDateStringFromDate(getDateFromMillis(millis));
    }
    
    private void toggleCheckBoxes(Node node, boolean isChecked) {
                        
        toggleChildren(node, isChecked);
        toggleParents(node, isChecked);        
        refreshRootNode();
    }

    private void toggleChildren(Node node, boolean isChecked) {

        List<Node> children = node.getChildren();
        for (Node child : children) {
            if (child instanceof CheckBoxTreeNode) {
                CheckBoxTreeNode childCheckBoxTreeNode = (CheckBoxTreeNode)child;
                childCheckBoxTreeNode.setChecked(isChecked);
            }

            if (!child.isLeaf()) {
                toggleChildren(child, isChecked);
            }
        }
    }

    private void toggleParents(Node node, boolean isChecked) {

        List<Node> parents = node.getParents();
        for (Node parent : parents) {
            if (parent instanceof DirectoryTreeNode) {

                boolean allChecked = true;
                if (isChecked) {
                    for (Node parentChild : parent.getChildren()) {
                        if (parentChild instanceof CheckBoxTreeNode) {
                            CheckBoxTreeNode parentChildCheckBoxTreeNode = (CheckBoxTreeNode)parentChild;
                            if (!parentChildCheckBoxTreeNode.isChecked()) {
                                allChecked = false;
                            }
                        }
                    }
                }

                if (allChecked) {
                    DirectoryTreeNode directoryTreeNode = (DirectoryTreeNode)parent;
                    directoryTreeNode.setChecked(isChecked);
                }
            }
        }
    }
}
