/*
 * DemoTreePanel.java
 *
 * Created on Nov 8, 2012, 7:26:00 PM
 */
package demoviewer.panels;

import demoviewer.common.Colors;
import demoviewer.gui.MainFrame;
import demoviewer.gui.dialogs.DemoFileChooser;
import demoviewer.model.Demo;
import demoviewer.model.Directory;
import demoviewer.model.label.ShadowLabel;
import demoviewer.model.tree.CheckBoxTreeNode;
import demoviewer.model.tree.DemoTreeNode;
import demoviewer.model.tree.DemoTreeTableModel;
import demoviewer.model.tree.DirectoryTreeNode;
import demoviewer.model.tree.ImageIconTreeNode;
import demoviewer.model.tree.Node;
import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.HeadlessException;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.Beans;
import java.io.File;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import javax.swing.DefaultCellEditor;
import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.ListSelectionModel;
import javax.swing.border.EmptyBorder;
import javax.swing.tree.DefaultTreeCellRenderer;
import org.jdesktop.swingx.JXTreeTable;
import org.jdesktop.swingx.decorator.ColorHighlighter;
import org.jdesktop.swingx.decorator.HighlightPredicate;
import org.jdesktop.swingx.decorator.Highlighter;

/**
 *
 * @author Me
 */
public class DemoTreePanel extends javax.swing.JPanel {
    
    private static ExecutorService threadPool = null;
    
    private JXTreeTable treeTable = null;    
    private boolean scanning = false;
    private boolean stopScanning = false;
    
    private ShadowLabel lblMessage = new ShadowLabel();
    
    private DisabledGlassPane disabledGlassPane = null;

    public DemoTreePanel() {
        initComponents();

        if (!Beans.isDesignTime()) {
            init();
        }
    }

    private void init() {
        initTreeTable();
        
        try {
            File file = new File("Z:\\RTCW\\demoViewer\\demos\\");
            Directory rootDir = new Directory(file);
            DirectoryTreeNode rootNode = new DirectoryTreeNode(rootDir);
            loadDemos(rootNode, file);
            setTreeTableModel(rootNode);
            
            jpScanning.add(lblMessage);
        }
        catch (SQLException | ClassNotFoundException ex) {
            MainFrame.the().showErrorMessageDialog("Failed to open demos.", ex);
        }
    }

    private void initTreeTable() {
        
        DemoTreeTableModel demoTreeTableModel = new DemoTreeTableModel(this);
        treeTable = new JXTreeTable(demoTreeTableModel);
        treeTable.setTreeTableModel(demoTreeTableModel);
        jpTableHolder.add(new JScrollPane(treeTable), BorderLayout.CENTER);

        treeTable.setRootVisible(true);
        treeTable.setShowsRootHandles(true);
        treeTable.setRowHeight(20);
        treeTable.setFocusable(false);
        treeTable.setSelectionBackground(Colors.COLOR_TABLE_ROW_SELECTED);

        treeTable.setDefaultEditor(Boolean.class, new TreeTableEditor());        
        treeTable.setTreeCellRenderer(new TreeTableRenderer());
        
        Highlighter highlightRowEven = new ColorHighlighter(HighlightPredicate.EVEN, Colors.COLOR_TABLE_ROW_EVEN, Color.BLACK);
        treeTable.addHighlighter(highlightRowEven);
        
        Highlighter highlightRowOdd = new ColorHighlighter(HighlightPredicate.EVEN, Colors.COLOR_TABLE_ROW_ODD, Color.BLACK);
        treeTable.addHighlighter(highlightRowOdd);
                
        treeTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        
        setTreeTableColumnWidths();
        initTreeTableListeners();
    }
    
    private void setTreeTableModel(Node rootNode) {
        
        DemoTreeTableModel demoTreeTableModel = new DemoTreeTableModel(rootNode, this);
        treeTable.setTreeTableModel(demoTreeTableModel); 
        setTreeTableColumnWidths();
        
        btnScanAdd.setEnabled(true);
        btnScan.setEnabled(true);
    }
    
    private void setTreeTableColumnWidths() {
        
        treeTable.getColumn(1).setPreferredWidth(100);
        treeTable.getColumn(1).setWidth(100);
        treeTable.getColumn(1).setMaxWidth(100);
        
        treeTable.getColumn(2).setPreferredWidth(60);
        treeTable.getColumn(2).setWidth(60);
        treeTable.getColumn(2).setMaxWidth(60);
        
        treeTable.getColumn(3).setPreferredWidth(80);
        treeTable.getColumn(3).setWidth(80);
        treeTable.getColumn(3).setMaxWidth(80);
        
        treeTable.getColumn(4).setMinWidth(30);
        treeTable.getColumn(4).setMaxWidth(30);        
    }

    private void initTreeTableListeners() {

        treeTable.addMouseListener(new MouseAdapter() {

            @Override
            public void mouseClicked(MouseEvent e) {

                int rowIndex = treeTable.rowAtPoint(e.getPoint());

                if (rowIndex >= 0 && rowIndex < treeTable.getRowCount()) {

                    Node selectedNode = (Node)treeTable.getPathForRow(rowIndex).getLastPathComponent();
                    if (selectedNode instanceof DemoTreeNode) {

                        DemoTreeNode demoTreeNode = (DemoTreeNode)selectedNode;
                        
                        txtDemoPath.setText(demoTreeNode.getDemo().getFile().getAbsolutePath());
                        
                        if (e.getButton() == MouseEvent.BUTTON1 && e.getClickCount() == 2) {
                            if (demoTreeNode.getDemo().isScanned()) {
                                viewDemo();
                            }
                            else {
                                List<DemoTreeNode> demoNodes = new ArrayList<>();
                                demoNodes.add(demoTreeNode);
                                scanDemos(demoNodes, true);                                
                            }
                        }
                    }
                    else if (selectedNode instanceof DirectoryTreeNode) {
                        DirectoryTreeNode directoryTreeNode = (DirectoryTreeNode)selectedNode;
                        txtDemoPath.setText(directoryTreeNode.getDirectory().getFile().getAbsolutePath());
                    }
                }
            }
        });
    }
    
    private void openDemos() {
        
        try {
            
            DemoFileChooser demoFileChooser = new DemoFileChooser();            
            int returnValue = demoFileChooser.showOpenDialog(MainFrame.the());
            if (returnValue == DemoFileChooser.APPROVE_OPTION) {
                
                File file = demoFileChooser.getSelectedFile();
                System.out.println(file);                
                
                if (file.isDirectory()) {
                    Directory rootDir = new Directory(file);
                    DirectoryTreeNode rootNode = new DirectoryTreeNode(rootDir);
                    loadDemos(rootNode, file);
                    setTreeTableModel(rootNode);
                }
            }
        }
        catch (HeadlessException | SQLException | ClassNotFoundException ex) {
            MainFrame.the().showErrorMessageDialog("Failed to open demos.", ex);
        }
    }
    
    private void loadDemos(Node node, File dir) throws SQLException, ClassNotFoundException {

        File[] list = dir.listFiles();

        for (File file : list) {
            if (file.isDirectory()) {
                Directory directory = new Directory(file);
                DirectoryTreeNode directoryTreeNode = new DirectoryTreeNode(directory);
                loadDemos(directoryTreeNode, file);
                node.add(directoryTreeNode);
            }
            else if (file.getAbsolutePath().endsWith(".dm_60")) {
                Demo demo = new Demo(file);
                Demo loadDemoFromDB = Demo.loadDemoFromDB(demo);
                if (loadDemoFromDB == null) {
                    DemoTreeNode demoTreeNode = new DemoTreeNode(demo);
                    node.add(demoTreeNode);
                }
                else {
                    DemoTreeNode demoTreeNode = new DemoTreeNode(loadDemoFromDB);
                    node.add(demoTreeNode);
                }
            }
        }
    }
    
    private void scanUnscannedDemos() {
        
        List<DemoTreeNode> unscannedDemoNodes = getUnscannedDemoNodes();
        
        if (unscannedDemoNodes.isEmpty()) {
            MainFrame.the().showErrorMessageDialog("No Demos Loaded.");
        }
        else {
            scanDemos(unscannedDemoNodes, false);
        }
    }

    private void scanSelectedDemos() {
        
        List<DemoTreeNode> selectedDemoNodes = getSelectedDemoNodes();
        
        if (selectedDemoNodes.isEmpty()) {
            MainFrame.the().showErrorMessageDialog("No Demos Selected.");
        }
        else {
            scanDemos(selectedDemoNodes, false);
        }
    }
    
    private void scanDemos(List<DemoTreeNode> demoNodes, final boolean viewDemo) {

        if (demoNodes != null && !demoNodes.isEmpty()) {
            
            CardLayout cl = (CardLayout)jpCardLayout.getLayout();
            cl.show(jpCardLayout, "scanning");
            
            disabledGlassPane = new DisabledGlassPane();
            MainFrame.the().setGlassPane(disabledGlassPane);
            disabledGlassPane.activate();

            threadPool = Executors.newFixedThreadPool(1);
            setScanning(true);
            refreshTreeFromRoot();

            try {

                int stepSize = 100 / demoNodes.size();
                for (int i = 0; i < demoNodes.size(); i++) {
                    DemoTreeNode demoTreeNode = demoNodes.get(i);
                    threadPool.submit(new DemoScanner(demoTreeNode, i, demoNodes.size(), stepSize));
                }
            }
            finally {

                threadPool.submit(new Runnable() {

                    @Override
                    public void run() {
                        scanDemosShutdown(false, viewDemo);
                    }
                });        
            }
        }
    }
    
    private void scanDemosShutdown(boolean stopped, boolean viewDemo) {
        
        if (stopped) {
            threadPool.shutdownNow();
        }
        else {
            threadPool.shutdown();
        }
        
        CardLayout cl = (CardLayout)jpCardLayout.getLayout();
        cl.show(jpCardLayout, "demoPath");        

        disabledGlassPane.deactivate();
        //MainFrame.the().setGlassPane(null);
        
        setScanning(false);
  
        treeTable.repaint();
        
        if (viewDemo) {
            viewDemo();
        }
    }
    
    private void stopScanningDemos() {

        if (isScanning()) {
            if (threadPool != null && !threadPool.isTerminated() && !threadPool.isShutdown()) {
                int returnValue = JOptionPane.showConfirmDialog(MainFrame.the(), "Are you sure you want to stop scanning demos?");
                if (returnValue == JOptionPane.YES_OPTION) {
                    setStopScanning(true);
                    scanDemosShutdown(true, false);
                    setStopScanning(false);
                }
            }
        }        
    }
    
    public List<DemoTreeNode> getUnscannedDemoNodes() {

        List<DemoTreeNode> demoNodes = new ArrayList<>();

        DemoTreeTableModel demoTreeTableModel = (DemoTreeTableModel)treeTable.getTreeTableModel();
        Node rootNode = (Node)demoTreeTableModel.getRoot();
        List<Node> allChildren = rootNode.getAllChildren();
        for (Node child : allChildren) {
            if (child instanceof DemoTreeNode) {
                DemoTreeNode demoTreeNode = (DemoTreeNode)child;
                if (!demoTreeNode.getDemo().isScanned()) {
                    demoNodes.add(demoTreeNode);
                }
            }
        }

        return demoNodes;
    }

    public List<DemoTreeNode> getSelectedDemoNodes() {

        List<DemoTreeNode> demoNodes = new ArrayList<>();

        DemoTreeTableModel demoTreeTableModel = (DemoTreeTableModel)treeTable.getTreeTableModel();
        Node rootNode = (Node)demoTreeTableModel.getRoot();
        List<Node> allChildren = rootNode.getAllChildren();
        for (Node child : allChildren) {
            if (child instanceof DemoTreeNode) {
                DemoTreeNode demoTreeNode = (DemoTreeNode)child;
                if (demoTreeNode.isChecked()) {
                    demoNodes.add(demoTreeNode);
                }
            }
        }

        return demoNodes;
    }
    
    public Node getSelectedNode() {
        
        int selectedRow = treeTable.getSelectedRow();
        if (selectedRow >= 0 && selectedRow < treeTable.getRowCount()) {
            return (Node) treeTable.getPathForRow(selectedRow).getLastPathComponent();
        }
        
        return null;        
    }
    
    private void viewDemo() {

        try {
            Node selectedNode = getSelectedNode();
            if (selectedNode != null && selectedNode instanceof DemoTreeNode) {
                DemoTreeNode demoTreeNode = (DemoTreeNode)selectedNode;
                demoTreeNode.getDemo().loadPlayersAndFrags();
                MainFrame.the().viewDemo(demoTreeNode.getDemo());
            }
        }
        catch (SQLException | ClassNotFoundException ex) {
            MainFrame.the().showErrorMessageDialog("Failed to load demo from database.", ex);
        }
    }

    public void refreshTree(Node node) {
        
        DemoTreeTableModel demoTreeTableModel = (DemoTreeTableModel)treeTable.getTreeTableModel();
        demoTreeTableModel.refreshNode(node);
    }

    public void refreshTreeFromRoot() {
        
        DemoTreeTableModel demoTreeTableModel = (DemoTreeTableModel)treeTable.getTreeTableModel();
        demoTreeTableModel.refreshRootNode();
    }
    
    public void setScanningMessage(DemoTreeNode demoTreeNode, int index, int total) {
        
        lblIcon.setIcon(DemoTreeNode.IMAGE_ICON_DEMO_SCANNING);
        lblMessage.setText(demoTreeNode.getText() + " [" + (index+1) + " of " + total + "]");        
    }
    
    public boolean isScanning() {
        return scanning;
    }

    public void setScanning(boolean scanning) {
        this.scanning = scanning;
        btnStopScanning.setEnabled(scanning);
    }
    
    public boolean isStopScanning() {
        return stopScanning;
    }

    public void setStopScanning(boolean stopScanning) {
        this.stopScanning = stopScanning;
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jpBorderLayout = new javax.swing.JPanel();
        jpToolbatHolder = new javax.swing.JPanel();
        jToolBar1 = new javax.swing.JToolBar();
        btnScanAdd = new javax.swing.JButton();
        btnScan = new javax.swing.JButton();
        btnStopScanning = new javax.swing.JButton();
        jpTableHolder = new javax.swing.JPanel();
        jpCardLayout = new javax.swing.JPanel();
        jpDemoPath = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        txtDemoPath = new javax.swing.JTextPane();
        jpScanning = new javax.swing.JPanel();
        lblIcon = new javax.swing.JLabel();

        setName("Form"); // NOI18N
        setLayout(new java.awt.BorderLayout());

        jpBorderLayout.setName("jpBorderLayout"); // NOI18N
        jpBorderLayout.setLayout(new java.awt.BorderLayout(0, 5));

        jpToolbatHolder.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, -5, 0));
        jpToolbatHolder.setName("jpToolbatHolder"); // NOI18N
        jpToolbatHolder.setPreferredSize(new java.awt.Dimension(10, 25));
        jpToolbatHolder.setLayout(new java.awt.BorderLayout());

        jToolBar1.setFloatable(false);
        jToolBar1.setRollover(true);
        jToolBar1.setName("jToolBar1"); // NOI18N
        jToolBar1.setPreferredSize(new java.awt.Dimension(71, 20));

        btnScanAdd.setIcon(new javax.swing.ImageIcon(getClass().getResource("/demoviewer/resources/icons/scanner_plus.png"))); // NOI18N
        btnScanAdd.setToolTipText("Scan Unscanned Demos");
        btnScanAdd.setEnabled(false);
        btnScanAdd.setFocusable(false);
        btnScanAdd.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        btnScanAdd.setName("btnScanAdd"); // NOI18N
        btnScanAdd.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        btnScanAdd.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnScanAddActionPerformed(evt);
            }
        });
        jToolBar1.add(btnScanAdd);

        btnScan.setIcon(new javax.swing.ImageIcon(getClass().getResource("/demoviewer/resources/icons/scanner.png"))); // NOI18N
        btnScan.setToolTipText("Scan Selected Demos");
        btnScan.setEnabled(false);
        btnScan.setFocusable(false);
        btnScan.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        btnScan.setName("btnScan"); // NOI18N
        btnScan.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        btnScan.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnScanActionPerformed(evt);
            }
        });
        jToolBar1.add(btnScan);

        btnStopScanning.setIcon(new javax.swing.ImageIcon(getClass().getResource("/demoviewer/resources/icons/scanner_cross.png"))); // NOI18N
        btnStopScanning.setToolTipText("Cancel Scanning Demos");
        btnStopScanning.setEnabled(false);
        btnStopScanning.setFocusable(false);
        btnStopScanning.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        btnStopScanning.setName("btnStopScanning"); // NOI18N
        btnStopScanning.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        btnStopScanning.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnStopScanningActionPerformed(evt);
            }
        });
        jToolBar1.add(btnStopScanning);

        jpToolbatHolder.add(jToolBar1, java.awt.BorderLayout.CENTER);

        jpBorderLayout.add(jpToolbatHolder, java.awt.BorderLayout.NORTH);

        jpTableHolder.setName("jpTableHolder"); // NOI18N
        jpTableHolder.setLayout(new java.awt.BorderLayout(0, 5));
        jpBorderLayout.add(jpTableHolder, java.awt.BorderLayout.CENTER);

        jpCardLayout.setName("jpCardLayout"); // NOI18N
        jpCardLayout.setPreferredSize(new java.awt.Dimension(0, 23));
        jpCardLayout.setVerifyInputWhenFocusTarget(false);
        jpCardLayout.setLayout(new java.awt.CardLayout());

        jpDemoPath.setName("jpDemoPath"); // NOI18N
        jpDemoPath.setLayout(new java.awt.BorderLayout());

        jScrollPane1.setName("jScrollPane1"); // NOI18N

        txtDemoPath.setName("txtDemoPath"); // NOI18N
        jScrollPane1.setViewportView(txtDemoPath);

        jpDemoPath.add(jScrollPane1, java.awt.BorderLayout.CENTER);

        jpCardLayout.add(jpDemoPath, "demoPath");

        jpScanning.setBackground(new java.awt.Color(64, 64, 64));
        jpScanning.setBorder(new JTextField().getBorder());
        jpScanning.setName("jpScanning"); // NOI18N
        jpScanning.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.CENTER, 5, 2));

        lblIcon.setBackground(new java.awt.Color(0, 0, 0));
        lblIcon.setName("lblIcon"); // NOI18N
        jpScanning.add(lblIcon);

        jpCardLayout.add(jpScanning, "scanning");

        jpBorderLayout.add(jpCardLayout, java.awt.BorderLayout.SOUTH);

        add(jpBorderLayout, java.awt.BorderLayout.CENTER);
    }// </editor-fold>//GEN-END:initComponents

private void btnScanActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnScanActionPerformed
    scanSelectedDemos();
}//GEN-LAST:event_btnScanActionPerformed

private void btnStopScanningActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnStopScanningActionPerformed
    stopScanningDemos();
}//GEN-LAST:event_btnStopScanningActionPerformed

    private void btnScanAddActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnScanAddActionPerformed
        scanUnscannedDemos();
    }//GEN-LAST:event_btnScanAddActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnScan;
    private javax.swing.JButton btnScanAdd;
    private javax.swing.JButton btnStopScanning;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JToolBar jToolBar1;
    private javax.swing.JPanel jpBorderLayout;
    private javax.swing.JPanel jpCardLayout;
    private javax.swing.JPanel jpDemoPath;
    private javax.swing.JPanel jpScanning;
    private javax.swing.JPanel jpTableHolder;
    private javax.swing.JPanel jpToolbatHolder;
    private javax.swing.JLabel lblIcon;
    private javax.swing.JTextPane txtDemoPath;
    // End of variables declaration//GEN-END:variables

    private class TreeTableRenderer extends DefaultTreeCellRenderer {

        public TreeTableRenderer() {

            setTextSelectionColor(Color.WHITE);
            setTextNonSelectionColor(Color.BLACK);
            setForeground(Color.BLACK);
            setBackground(new Color(0, 0, 0, 0));
            setBackgroundNonSelectionColor(new Color(0, 0, 0, 0));
            setBackgroundSelectionColor(new Color(0, 0, 0, 0));
        }

        @Override
        public Component getTreeCellRendererComponent(JTree tree, Object value, boolean isSelected, boolean isExpanded, boolean isLeaf, int rowIndex, boolean hasFocus) {

            super.getTreeCellRendererComponent(tree, value, isSelected, isExpanded, isLeaf, rowIndex, hasFocus);

            ImageIconTreeNode imageIconTreeNode = (ImageIconTreeNode)value;
            ImageIcon imageIcon = imageIconTreeNode.getImageIcon();

            if (value instanceof DemoTreeNode) {
                DemoTreeNode demoTreeNode = (DemoTreeNode)value;

                if (demoTreeNode.getDemo().isScanning()) {
                    imageIcon = DemoTreeNode.IMAGE_ICON_DEMO_SCANNING;
                }

                if (demoTreeNode.getDemo().isScanned()) {
                    imageIcon = DemoTreeNode.IMAGE_ICON_DEMO_SCANNED;
                }
            }

            FlowLayout flowLayout = new FlowLayout();
            flowLayout.setHgap(5);
            flowLayout.setVgap(0);

            JPanel wrapper = new JPanel();
            wrapper.setLayout(new BorderLayout());
            wrapper.setOpaque(false);

            JPanel panel = new JPanel();
            panel.setLayout(flowLayout);
            panel.setBorder(new EmptyBorder(2, -3, 2, -3));

            JLabel label = new JLabel(imageIconTreeNode.getText());
            if (isSelected) {
                label.setForeground(Color.WHITE);
            }
            else {
                label.setForeground(Color.BLACK);
            }

            panel.setOpaque(false);

            JLabel imageIconLabel = new JLabel(imageIcon);

            panel.add(imageIconLabel);
            panel.add(label);
            
            wrapper.add(panel, BorderLayout.CENTER);

            return wrapper;
        }
    }

    private static class TreeTableEditor extends DefaultCellEditor {

        private JCheckBox checkBox = null;
        private CheckBoxTreeNode checkBoxTreeNode = null;

        public TreeTableEditor() {
            super(new JCheckBox());
            this.checkBox = (JCheckBox)getComponent();
            checkBox.setHorizontalAlignment(JCheckBox.CENTER);
            checkBox.setBackground(Colors.COLOR_TABLE_ROW_SELECTED);
            checkBox.setFocusable(false);
        }

        @Override
        public Object getCellEditorValue() {

            if (checkBoxTreeNode == null) {
                return false;
            }

            return !checkBoxTreeNode.isChecked();
        }

        @Override
        public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int rowIndex, int columnIndex) {

            Object valueAt = table.getValueAt(rowIndex, 0);
            if (valueAt instanceof CheckBoxTreeNode) {
                checkBoxTreeNode = (CheckBoxTreeNode)valueAt;
                checkBox.setSelected(!checkBoxTreeNode.isChecked());
            }
            
            return checkBox;
        }
    }
    
    private class DemoScanner implements Callable {
   
        private DemoTreeNode demoTreeNode = null;        
        private int index = -1;
        private int total = -1;
        private int stepSize = -1;

        public DemoScanner(DemoTreeNode demoTreeNode, int index, int total, int stepSize) {

            this.demoTreeNode = demoTreeNode;
            this.index = index;
            this.total = total;
            this.stepSize = stepSize;
            
            initScanning();
        }
        
        private void initScanning() {
            
            demoTreeNode.getDemo().setScanning(true);
            demoTreeNode.getDemo().setScanned(false);
            treeTable.repaint();
        }

        @Override
        public Object call() throws Exception {
                        
            try {
                
                //progressBar.setString("Scanning Demo: " + (index + 1) + " of " + total);
                //glassPane.setMessage("Scanning Demo: " + (index + 1) + " of " + total);
                setScanningMessage(demoTreeNode, index, total);
                                
                Process p = Runtime.getRuntime().exec("DemoParser.exe " + demoTreeNode.getDemo().getFile().getAbsolutePath());
                p.waitFor();
                
                demoTreeNode.getDemo().setScanning(false);

                if (p.exitValue() == 0) {
                    demoTreeNode.getDemo().setScanned(true);
                    Demo loadDemoFromDB = Demo.loadDemoFromDB(demoTreeNode.getDemo());
                    demoTreeNode.setDemo(loadDemoFromDB);
                }
                else {
                    demoTreeNode.getDemo().setScanned(false);
                }
                
                treeTable.repaint();
                                
                return p.exitValue();
            }
            catch (InterruptedException ex) {
                if (!isStopScanning()) {
                    MainFrame.the().showErrorMessageDialog("Demo Scanning Interrupted.", ex);
                }
            }
            catch (Exception ex) {
                MainFrame.the().showErrorMessageDialog("Scanning Demo Failed.", ex);
            }
            
            return null;
        }
    }
}
