/*
 * PlayerTablePanel.java
 *
 * Created on Nov 24, 2012, 12:20:03 PM
 */
package demoviewer.panels;

import demoviewer.common.Colors;
import demoviewer.common.Layouts;
import demoviewer.gui.MainFrame;
import demoviewer.model.Demo;
import demoviewer.model.Frag;
import demoviewer.model.Player;
import demoviewer.model.label.ShadowLabel;
import demoviewer.model.tables.FragTableModel;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.List;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListCellRenderer;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.RowFilter;
import javax.swing.UIManager;
import javax.swing.border.EmptyBorder;
import javax.swing.plaf.basic.BasicComboBoxRenderer;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableRowSorter;

/**
 *
 * @author Me
 */
public class FragTablePanel extends javax.swing.JPanel {
    
    public static final ImageIcon IMAGE_ICON_POV = new ImageIcon(FragTablePanel.class.getResource("/demoviewer/resources/icons/eye.png"));
            
    private Demo demo = null;

    private FragTablePanel() {
        initComponents();
    }

    public FragTablePanel(Demo demo) {

        this();

        this.demo = demo;

        init();
    }

    private void init() {

        initTable();
        initPlayerComobBox();
    }

    private void initTable() {
        
        FragTableModel fragTableModel = new FragTableModel(demo.getFrags());
        table.setModel(fragTableModel);
        table.setRowHeight(20);
        table.setDefaultRenderer(String.class, new DefaultStringRenderer());
        table.setShowGrid(true);
        table.setIntercellSpacing(new Dimension(0, 0));
        
        table.addMouseListener(new MouseAdapter() {

            @Override
            public void mouseClicked(MouseEvent e) {

                int rowIndex = table.rowAtPoint(e.getPoint());
                if (rowIndex >= 0 && rowIndex < table.getRowCount() && e.getClickCount() == 2) {
                    
                    FragTableModel fragTableModel = (FragTableModel)table.getModel();
                    
                    int selectStartIndex = rowIndex;
                    int selectEndIndex = rowIndex;
                    
                    for (int i = rowIndex; i >= 0; i--) {                        
                        Frag frag = fragTableModel.getFrags().get(table.convertRowIndexToModel(i));
                        if (frag.isLastFragInSpree()) {
                            if (i != rowIndex) {
                                selectStartIndex = i + 1;
                                i = -1;
                            }                            
                        }  
                        else if (i == 0) {
                            selectStartIndex = i;
                        }
                    }
                    
                    for (int i = rowIndex; i < table.getRowCount(); i++) {                        
                        Frag frag = fragTableModel.getFrags().get(table.convertRowIndexToModel(i));
                        if (frag.isLastFragInSpree()) {
                            selectEndIndex = i;
                            i = table.getRowCount();
                        }
                        else if (i == table.getRowCount() - 1) {
                            selectEndIndex = i;
                        }
                    }
                    
                    table.getSelectionModel().setSelectionInterval(selectStartIndex, selectEndIndex);
                }
            }
        });

        setTableColumnWidths();

        hideFromTable("Attacker ID");
        hideFromTable("Target ID");
        hideFromTable("Weapon");
        hideFromTable("Team Kill");
        hideFromTable("Frag ID");
        hideFromTable("Last In Spree");
        
        table.setTableHeader(null);
    }

    public void hideFromTable(String columnName) {

        int index = table.getColumnModel().getColumnIndex(columnName);
        TableColumn column = table.getColumnModel().getColumn(index);
        table.getColumnModel().removeColumn(column);
    }
    
    private void setTableColumnWidths() {

        table.getColumnModel().getColumn(0).setPreferredWidth(50);
        table.getColumnModel().getColumn(0).setWidth(50);
        table.getColumnModel().getColumn(0).setMaxWidth(50);
    }
    
    private void initPlayerComobBox() {
            
        int demoPovIndex = 0;
        
        DefaultComboBoxModel defaultComboBoxModel = new DefaultComboBoxModel();
        for (int i = 0; i < demo.getPlayers().size(); i++) {
            
            Player player = demo.getPlayers().get(i);
        
            defaultComboBoxModel.addElement(player);
            if (demo.isPov(player)) {
                demoPovIndex = i;
            }
        } 
        
        cbPlayer.setModel(defaultComboBoxModel);
        cbPlayer.setRenderer(new ComboBoxRenderer());
        cbPlayer.setBackground(Colors.COLOR_TABLE_ROW_NOT_SELECTED);
        
        if (cbPlayer.getModel().getSize() > 0) {
            cbPlayer.setSelectedIndex(demoPovIndex);
        }
    }
    
    public void setFilterMode(boolean filter) {
        
        if (filter) {            
            cbPlayer.setVisible(true);
            filterFrags();            
        }
        else {
            cbPlayer.setVisible(false);
            clearFilters(); 
        }
    }

    private void filterFrags() {

        if (MainFrame.the().isFilter()) {

            FragTableModel fragTableModel = (FragTableModel)table.getModel();
            TableRowSorter<FragTableModel> sorter = new TableRowSorter<>(fragTableModel);

            RowFilter<Object, Object> spreeFilter = getSpreeFilters();
            List<RowFilter<Object, Object>> filters = new ArrayList<>();
            filters.add(spreeFilter);

            RowFilter<Object, Object> filter = RowFilter.andFilter(filters);
            sorter.setRowFilter(filter);

            table.setRowSorter(sorter);

            fragTableModel.fireTableDataChanged();
        }
    }
    
    public RowFilter<Object, Object> getSpreeFilters() {
        
        int minSpree = MainFrame.the().getMinSpree();
        Player player = (Player)cbPlayer.getSelectedItem();

        List<RowFilter<Object, Object>> filters = new ArrayList<>();
        
        List<RowFilter<Object, Object>> fragIdFilters = new ArrayList<>();
        
        FragTableModel fragTableModel = (FragTableModel)table.getModel();        
        int lastFragRoundTime = -1;
        int lastSpreeFragRoundTime = -1;
        int timeBetweenFrags = MainFrame.the().getTimeBetweenFrags() * 1000;
        int lastAddedFragIndex = -1;
        
        for (int i = 0; i < fragTableModel.getFrags().size(); i++) {
            
            Frag frag = fragTableModel.getFrags().get(i);
            frag.setLastFragInSpree(false);
 
            if (i == fragTableModel.getFrags().size() - 1) {
                if (addSpree(minSpree, frag, filters, fragIdFilters, true)) {
                    frag.setLastFragInSpree(true);
                }
                fragIdFilters = new ArrayList<>();
                lastSpreeFragRoundTime = -1;
                lastAddedFragIndex = -1;
            }
            else if (frag.getTargetId() == player.getPlayerId()) {
                if (addSpree(minSpree, frag, filters, fragIdFilters, true)) {
                    frag.setLastFragInSpree(true);
                }
                fragIdFilters = new ArrayList<>();
                lastSpreeFragRoundTime = -1;
                lastAddedFragIndex = -1;
            }
            else if (lastFragRoundTime != -1 && frag.getRoundTime() > lastFragRoundTime) {
                if (addSpree(minSpree, frag, filters, fragIdFilters, true)) {
                    frag.setLastFragInSpree(true);
                }
                fragIdFilters = new ArrayList<>();
                lastSpreeFragRoundTime = -1;
                lastAddedFragIndex = -1;
            }
            else if (frag.getAttackerId() == player.getPlayerId() && frag.getTeamKill() == Frag.TEAM_KILL_NO && lastSpreeFragRoundTime != -1 && ((lastSpreeFragRoundTime - frag.getRoundTime()) > timeBetweenFrags)) {
                if (addSpree(minSpree, frag, filters, fragIdFilters, false)) {                    
                    Frag lastAddedFrag = fragTableModel.getFrags().get(lastAddedFragIndex);
                    lastAddedFrag.setLastFragInSpree(true);                 
                }
                fragIdFilters = new ArrayList<>();
                lastSpreeFragRoundTime = -1;   
                lastAddedFragIndex = -1;
                i--;
            }
            else if (frag.getAttackerId() == player.getPlayerId() && frag.getTeamKill() == Frag.TEAM_KILL_NO && (lastSpreeFragRoundTime != -1 || ((lastSpreeFragRoundTime - frag.getRoundTime()) <= timeBetweenFrags))) {
                RowFilter<Object, Object> fragIdFilter = RowFilter.numberFilter(RowFilter.ComparisonType.EQUAL, frag.getId(), FragTableModel.COLUMN_FRAG_ID);
                fragIdFilters.add(fragIdFilter);  
                lastSpreeFragRoundTime = (int)frag.getRoundTime();
                lastAddedFragIndex = i;
            }            
            
            lastFragRoundTime = (int)frag.getRoundTime();
        }
        
        RowFilter<Object, Object> filter = RowFilter.orFilter(filters);        
        
        return filter;        
    }
    
    private boolean addSpree(int minSpree, Frag frag, List<RowFilter<Object, Object>> filters, List<RowFilter<Object, Object>> fragIdFilters, boolean addFrag) {
        
        if (includeSpree(minSpree, fragIdFilters)) {  
            
            if (addFrag) {
                RowFilter<Object, Object> fragIdFilter = RowFilter.numberFilter(RowFilter.ComparisonType.EQUAL, frag.getId(), FragTableModel.COLUMN_FRAG_ID);
                fragIdFilters.add(fragIdFilter);
            }
            filters.addAll(fragIdFilters);
            return true;
        } 
        
        return false;
    }
    
    private boolean includeSpree(int minSpree, List<RowFilter<Object, Object>> fragIdFilters) {
        
        if (fragIdFilters.size() >= minSpree) {
            return true;
        }

        return false;        
    }
    
    private void clearFilters() {

        FragTableModel fragTableModel = (FragTableModel)table.getModel();
        for (Frag frag : fragTableModel.getFrags()) {
            frag.setLastFragInSpree(false);
        }

        TableRowSorter<FragTableModel> sorter = new TableRowSorter<>(fragTableModel);
        sorter.setRowFilter(null);
        table.setRowSorter(sorter);

        fragTableModel.fireTableDataChanged();
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jpBorderLayout = new javax.swing.JPanel();
        jpToolbatHolder = new javax.swing.JPanel();
        jToolBar1 = new javax.swing.JToolBar();
        btnAddSpree = new javax.swing.JButton();
        jpPlayer = new javax.swing.JPanel();
        cbPlayer = new javax.swing.JComboBox();
        jpTableHolder = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        table = new javax.swing.JTable();

        setBorder(javax.swing.BorderFactory.createEmptyBorder(5, 5, 5, 5));
        setLayout(new java.awt.BorderLayout());

        jpBorderLayout.setLayout(new java.awt.BorderLayout(0, 7));

        jpToolbatHolder.setPreferredSize(new java.awt.Dimension(10, 30));
        jpToolbatHolder.setLayout(new java.awt.BorderLayout());

        jToolBar1.setFloatable(false);
        jToolBar1.setRollover(true);

        btnAddSpree.setIcon(new javax.swing.ImageIcon(getClass().getResource("/demoviewer/resources/icons/add_film.png"))); // NOI18N
        btnAddSpree.setToolTipText("Select All Filters");
        btnAddSpree.setFocusable(false);
        btnAddSpree.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        btnAddSpree.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jToolBar1.add(btnAddSpree);

        jpToolbatHolder.add(jToolBar1, java.awt.BorderLayout.CENTER);

        jpPlayer.setBorder(javax.swing.BorderFactory.createEmptyBorder(2, 0, 2, 0));
        jpPlayer.setPreferredSize(new java.awt.Dimension(250, 30));
        jpPlayer.setLayout(new java.awt.BorderLayout());

        cbPlayer.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
        cbPlayer.setPreferredSize(new java.awt.Dimension(200, 20));
        cbPlayer.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                cbPlayerItemStateChanged(evt);
            }
        });
        jpPlayer.add(cbPlayer, java.awt.BorderLayout.CENTER);

        jpToolbatHolder.add(jpPlayer, java.awt.BorderLayout.EAST);

        jpBorderLayout.add(jpToolbatHolder, java.awt.BorderLayout.NORTH);

        jpTableHolder.setLayout(new java.awt.BorderLayout());

        table.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4"
            }
        ));
        jScrollPane1.setViewportView(table);

        jpTableHolder.add(jScrollPane1, java.awt.BorderLayout.CENTER);

        jpBorderLayout.add(jpTableHolder, java.awt.BorderLayout.CENTER);

        add(jpBorderLayout, java.awt.BorderLayout.CENTER);
    }// </editor-fold>//GEN-END:initComponents

    private void cbPlayerItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_cbPlayerItemStateChanged
        filterFrags();
    }//GEN-LAST:event_cbPlayerItemStateChanged

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnAddSpree;
    private javax.swing.JComboBox cbPlayer;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JToolBar jToolBar1;
    private javax.swing.JPanel jpBorderLayout;
    private javax.swing.JPanel jpPlayer;
    private javax.swing.JPanel jpTableHolder;
    private javax.swing.JPanel jpToolbatHolder;
    private javax.swing.JTable table;
    // End of variables declaration//GEN-END:variables

    private class DefaultStringRenderer extends DefaultTableCellRenderer {

        @Override
        public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int rowIndex, int columnIndex) {

            if (value instanceof String) {

                JPanel panel = new JPanel();                
                panel.setLayout(Layouts.LAYOUT_CONSOLE);

                if (isSelected) {
                    panel.setBackground(Colors.COLOR_TABLE_ROW_SELECTED);
                }
                else {
                    panel.setBackground(Colors.COLOR_TABLE_ROW_NOT_SELECTED);
                }

                ShadowLabel shadowLabel = new ShadowLabel();
                shadowLabel.setText((String)value);
                
                panel.add(shadowLabel);
                
                FragTableModel fragTableModel = (FragTableModel)table.getModel();
                Boolean lastInSpree = (Boolean)fragTableModel.getValueAt(table.convertRowIndexToModel(rowIndex), FragTableModel.COLUMN_LAST_IN_SPREE);
                if (lastInSpree) {                    
                    JPanel panelWrapper = new JPanel();
                    panelWrapper.setBackground(UIManager.getColor("Table.gridColor"));
                    panelWrapper.setLayout(new BorderLayout());
                    panelWrapper.setBorder(new EmptyBorder(0, 0, 1, 0));
                    panelWrapper.add(panel, BorderLayout.CENTER);
                    return panelWrapper;                    
                }
                
                return panel;
            }

            return super.getTableCellRendererComponent(table, value, isSelected, hasFocus, rowIndex, columnIndex);
        }
    }

    private class ComboBoxRenderer extends BasicComboBoxRenderer {

        protected DefaultListCellRenderer defaultRenderer = new DefaultListCellRenderer();

        @Override
        public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean hasFocus) {

            if (value instanceof Player) {
                
                JPanel panel = new JPanel();                
                panel.setLayout(Layouts.LAYOUT_CONSOLE);
                
                Player player = (Player)value;
                                
                String playerName = player.getPlayerName();
                ShadowLabel shadowLabel = new ShadowLabel();
                shadowLabel.setText(playerName);
                                
                panel.add(shadowLabel);

                if (isSelected || hasFocus) {
                    panel.setBackground(Colors.COLOR_TABLE_ROW_SELECTED);
                }
                else if(demo.isPov(player)) {
                    panel.setBackground(Colors.COLOR_TABLE_ROW_POV);
                }
                else {
                    panel.setBackground(Colors.COLOR_TABLE_ROW_NOT_SELECTED);
                }
                
                return panel;
            }
            
            return super.getListCellRendererComponent(list, value, index, isSelected, hasFocus);
        }
    }
}
