/*
 * PlayerTablePanel.java
 *
 * Created on Nov 24, 2012, 12:20:03 PM
 */
package demoviewer.panels;

import demoviewer.common.Colors;
import demoviewer.model.Demo;
import demoviewer.model.Player;
import demoviewer.model.tables.PlayerTableModel;
import java.awt.Component;
import java.util.ArrayList;
import java.util.List;
import javax.swing.DefaultCellEditor;
import javax.swing.JCheckBox;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.table.DefaultTableCellRenderer;

/**
 *
 * @author Me
 */
public class PlayerTablePanel extends javax.swing.JPanel {

    private Demo demo = null;

    private PlayerTablePanel() {
        initComponents();
    }

    public PlayerTablePanel(Demo demo) {

        this();

        this.demo = demo;

        init();
    }

    private void init() {
        initTable();
    }

    private void initTable() {

        PlayerTableModel playerTableModel = new PlayerTableModel(demo.getPlayers());
        table.setModel(playerTableModel);
        table.setRowHeight(30);
        table.setBackground(Colors.COLOR_TABLE_ROW_NOT_SELECTED);
        table.setSelectionBackground(Colors.COLOR_TABLE_ROW_SELECTED);
        table.setFocusable(false);
        table.setDefaultRenderer(String.class, new DefaultStringRenderer());
        table.setDefaultEditor(Boolean.class, new DefaultCheckBoxEditor());
        table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        
        setTableColumnWidths();
    }
    
    private void setTableColumnWidths() {
        
        table.getColumnModel().getColumn(0).setMinWidth(30);
        table.getColumnModel().getColumn(0).setMaxWidth(30);
        
        table.getColumnModel().getColumn(2).setMinWidth(100);
        table.getColumnModel().getColumn(2).setMaxWidth(100);   
        
        table.getColumnModel().getColumn(3).setMinWidth(30);
        table.getColumnModel().getColumn(3).setMaxWidth(30);     
    }
    
    public List<Player> getSelectedPlayers() {
        
        List<Player> selectedPlayers = new ArrayList<>();

        PlayerTableModel playerTableModel = (PlayerTableModel) table.getModel();
        for (Player player : playerTableModel.getPlayers()) {
            if (player.isSelected()) {
                selectedPlayers.add(player);
            }
        }   
        
        return selectedPlayers;
    }

    private void setSelectedAll(boolean selected) {
        
        PlayerTableModel playerTableModel = (PlayerTableModel) table.getModel();       
        for (Player player : playerTableModel.getPlayers()) {
            player.setSelected(selected);
        }
        
        playerTableModel.fireTableDataChanged(); 
    }
        
    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jpBorderLayout = new javax.swing.JPanel();
        jpTableHolder = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        table = new javax.swing.JTable();
        jpToolbatHolder = new javax.swing.JPanel();
        jToolBar1 = new javax.swing.JToolBar();
        btnSelectAll = new javax.swing.JButton();
        btnSelectNone = new javax.swing.JButton();

        setLayout(new java.awt.BorderLayout());

        jpBorderLayout.setLayout(new java.awt.BorderLayout(0, 3));

        jpTableHolder.setLayout(new java.awt.BorderLayout());

        table.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4"
            }
        ));
        jScrollPane1.setViewportView(table);

        jpTableHolder.add(jScrollPane1, java.awt.BorderLayout.CENTER);

        jpBorderLayout.add(jpTableHolder, java.awt.BorderLayout.CENTER);

        jpToolbatHolder.setBorder(javax.swing.BorderFactory.createCompoundBorder(new JTextField().getBorder(), javax.swing.BorderFactory.createEmptyBorder(2, 5, 2, 5)));
        jpToolbatHolder.setPreferredSize(new java.awt.Dimension(10, 30));
        jpToolbatHolder.setLayout(new java.awt.BorderLayout());

        jToolBar1.setFloatable(false);
        jToolBar1.setRollover(true);

        btnSelectAll.setIcon(new javax.swing.ImageIcon(getClass().getResource("/demoviewer/resources/icons/check_all.png"))); // NOI18N
        btnSelectAll.setToolTipText("Select All");
        btnSelectAll.setFocusable(false);
        btnSelectAll.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        btnSelectAll.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        btnSelectAll.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnSelectAllActionPerformed(evt);
            }
        });
        jToolBar1.add(btnSelectAll);

        btnSelectNone.setIcon(new javax.swing.ImageIcon(getClass().getResource("/demoviewer/resources/icons/check_none.png"))); // NOI18N
        btnSelectNone.setToolTipText("Deselect All");
        btnSelectNone.setFocusable(false);
        btnSelectNone.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        btnSelectNone.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        btnSelectNone.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnSelectNoneActionPerformed(evt);
            }
        });
        jToolBar1.add(btnSelectNone);

        jpToolbatHolder.add(jToolBar1, java.awt.BorderLayout.CENTER);

        jpBorderLayout.add(jpToolbatHolder, java.awt.BorderLayout.NORTH);

        add(jpBorderLayout, java.awt.BorderLayout.CENTER);
    }// </editor-fold>//GEN-END:initComponents

    private void btnSelectAllActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnSelectAllActionPerformed
        setSelectedAll(true);
    }//GEN-LAST:event_btnSelectAllActionPerformed

    private void btnSelectNoneActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnSelectNoneActionPerformed
        setSelectedAll(false);
    }//GEN-LAST:event_btnSelectNoneActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnSelectAll;
    private javax.swing.JButton btnSelectNone;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JToolBar jToolBar1;
    private javax.swing.JPanel jpBorderLayout;
    private javax.swing.JPanel jpTableHolder;
    private javax.swing.JPanel jpToolbatHolder;
    private javax.swing.JTable table;
    // End of variables declaration//GEN-END:variables

    private class DefaultStringRenderer extends DefaultTableCellRenderer {
        
        @Override
        public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int rowIndex, int columnIndex) {
                  
            if (value instanceof String) {
                
                String str = (String) value;
                ColorTextPanel colorTextPanel = new ColorTextPanel(str);
                
                if (isSelected) {
                    colorTextPanel.setBackground(Colors.COLOR_TABLE_ROW_SELECTED);
                } else {
                    colorTextPanel.setBackground(Colors.COLOR_TABLE_ROW_NOT_SELECTED);
                }

                return colorTextPanel;
            }           
            
            return super.getTableCellRendererComponent(table, value, isSelected, hasFocus, rowIndex, columnIndex);
        }     
    }
    
    private static class DefaultCheckBoxEditor extends DefaultCellEditor {

        private JCheckBox checkBox = null;

        public DefaultCheckBoxEditor() {
            super(new JCheckBox());
            this.checkBox = (JCheckBox)getComponent();
        }

        @Override
        public Object getCellEditorValue() {
            return checkBox.isSelected();
        }

        @Override
        public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int rowIndex, int columnIndex) { 

            checkBox = (JCheckBox)super.getTableCellEditorComponent(table, value, isSelected, rowIndex, columnIndex);
            
            checkBox.setHorizontalAlignment(JCheckBox.CENTER);
            checkBox.setBackground(Colors.COLOR_TABLE_ROW_SELECTED);
            checkBox.setFocusable(false);
            
            return checkBox;
        }
    }
}
