
package demoviewer.panels;

import demoviewer.gui.MainFrame;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTabbedPane;

/**
 *
 * @author Me
 */
public class TabButtonPanel extends javax.swing.JPanel {
    
    private JTabbedPane tabbedPane = null;
    private int index = -1;
    private boolean closeable = false;
    
    private TabButtonPanel() {
        initComponents();
    }
    
    public TabButtonPanel(JTabbedPane tabbedPane, int index, boolean closeable) {
        
        this();
        
        this.tabbedPane = tabbedPane;
        this.index = index;
        this.closeable = closeable;
        
        init();
    }
    
    private void init() {   
        
        if (!closeable) {            
            jpToolbarHolder.setVisible(false);
        }
                
        lblTitle.setText(tabbedPane.getTitleAt(index));  
        
        Dimension size = new Dimension(getPreferredSize().width, 18);
        setPreferredSize(size);
        
        btnClose.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseClicked(MouseEvent e) {             
                close();
            }
        });
        
        addMouseListener(new MouseAdapter() {
            
            @Override
            public void mouseReleased(MouseEvent e) {
                
                int indexOfTabComponent = tabbedPane.indexOfTabComponent(TabButtonPanel.this); 
                if (e.isPopupTrigger() && indexOfTabComponent >= MainFrame.TAB_COUNT_DEFAULT && tabbedPane.getSelectedIndex() == indexOfTabComponent) {
                    PopupMenu popupMenu = new PopupMenu();
                    popupMenu.show(e.getComponent(), e.getX(), e.getY());
                }
            }

            @Override
            public void mouseClicked(MouseEvent e) {
                
                if (e.getButton() == MouseEvent.BUTTON1) {
                    int indexOfTabComponent = tabbedPane.indexOfTabComponent(TabButtonPanel.this); 
                    tabbedPane.setSelectedIndex(indexOfTabComponent);
                }
            }
        });
    }
    
    private void close() {
        
        int indexOfTabComponent = tabbedPane.indexOfTabComponent(TabButtonPanel.this); 
        tabbedPane.removeTabAt(indexOfTabComponent);
        if (tabbedPane.getTabCount() <= MainFrame.TAB_COUNT_DEFAULT) {
            tabbedPane.setSelectedIndex(0);
        }
    }
    
    private void closeAll() {
                                
        for (int i = tabbedPane.getTabCount() - 1; i >= MainFrame.TAB_COUNT_DEFAULT; i--) {
            tabbedPane.removeTabAt(i); 
            System.out.println(i);
        }
        
        tabbedPane.setSelectedIndex(0);
    }
    
    private void closeOthers() {
                
        int indexOfTabComponent = tabbedPane.indexOfTabComponent(TabButtonPanel.this);         
        for (int i = tabbedPane.getTabCount() - 1; i >= MainFrame.TAB_COUNT_DEFAULT; i--) {
            
            if (i != indexOfTabComponent) {
                tabbedPane.removeTabAt(i);  
            }         
        }
    }
    
    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        lblTitle = new javax.swing.JLabel();
        jpToolbarHolder = new javax.swing.JPanel();
        toolbar = new javax.swing.JToolBar();
        btnClose = new javax.swing.JButton();

        setMaximumSize(new java.awt.Dimension(32767, 20));
        setMinimumSize(new java.awt.Dimension(0, 20));
        setOpaque(false);
        setLayout(new java.awt.BorderLayout());

        lblTitle.setText("Title");
        add(lblTitle, java.awt.BorderLayout.CENTER);

        jpToolbarHolder.setBorder(javax.swing.BorderFactory.createEmptyBorder(2, 0, 0, 0));
        jpToolbarHolder.setMaximumSize(new java.awt.Dimension(16, 16));
        jpToolbarHolder.setMinimumSize(new java.awt.Dimension(16, 16));
        jpToolbarHolder.setOpaque(false);
        jpToolbarHolder.setPreferredSize(new java.awt.Dimension(16, 16));
        jpToolbarHolder.setLayout(new java.awt.BorderLayout());

        toolbar.setBorder(null);
        toolbar.setFloatable(false);
        toolbar.setRollover(true);
        toolbar.setMaximumSize(new java.awt.Dimension(16, 16));
        toolbar.setMinimumSize(new java.awt.Dimension(16, 16));
        toolbar.setOpaque(false);
        toolbar.setPreferredSize(new java.awt.Dimension(18, 18));

        btnClose.setIcon(new javax.swing.ImageIcon(getClass().getResource("/demoviewer/resources/icons/close.png"))); // NOI18N
        btnClose.setFocusable(false);
        btnClose.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        btnClose.setMaximumSize(new java.awt.Dimension(16, 16));
        btnClose.setMinimumSize(new java.awt.Dimension(16, 16));
        btnClose.setOpaque(false);
        btnClose.setPreferredSize(new java.awt.Dimension(16, 16));
        btnClose.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        toolbar.add(btnClose);

        jpToolbarHolder.add(toolbar, java.awt.BorderLayout.CENTER);

        add(jpToolbarHolder, java.awt.BorderLayout.EAST);
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnClose;
    private javax.swing.JPanel jpToolbarHolder;
    private javax.swing.JLabel lblTitle;
    private javax.swing.JToolBar toolbar;
    // End of variables declaration//GEN-END:variables

    private class PopupMenu extends JPopupMenu implements ActionListener {

        public PopupMenu() {
            init();
        }
        
        private void init() {
            
            JMenuItem jmiClose = new JMenuItem("Close");
            JMenuItem jmiCloseAll = new JMenuItem("Close All");
            JMenuItem jmiCloseOthers = new JMenuItem("Close Others");

            jmiClose.addActionListener(this);
            jmiCloseAll.addActionListener(this);
            jmiCloseOthers.addActionListener(this);

            add(jmiClose);
            add(jmiCloseAll);
            add(jmiCloseOthers);
        }

        @Override
        public void actionPerformed(ActionEvent event) {
            
            switch (event.getActionCommand()) {
                case "Close":
                    close();
                    break;
                case "Close All":
                    closeAll();
                    break;
                case "Close Others":
                    closeOthers();
                    break;                
            }
        }
    }
}
